package es.imib.export;

import es.imib.persistence.MedicationSubstance;
import es.imib.persistence.Study;
import es.imib.persistence.Therapy;
import es.imib.persistence.daos.MedicationSubstanceDAO;
import es.imib.persistence.daos.StudyDAO;
import java.util.Date;
import java.util.List;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLIndividual;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class TherapyBean extends OWLMigrationBean<Therapy> implements Comparable<TherapyBean> {

    protected Study study;
    protected List<MedicationSubstance> medications;
    protected StudyDAO studyDAO;
    protected MedicationSubstanceDAO medicationSubstanceDAO;
    protected PatientBean patientBean;

    public TherapyBean(Therapy bean, PatientBean patientBean) {
        super(bean);
        studyDAO = StudyDAO.getStudyDAO();
        medicationSubstanceDAO = MedicationSubstanceDAO.getMedicationSubstanceDAO();
        medications = medicationSubstanceDAO.findByTherapy(bean.getMeasureNdx(), null, false);
        if (bean.getStudyNdx() != null) {
            study = studyDAO.findById(bean.getStudyNdx());
        }
        this.patientBean = patientBean;
    }

    public Study getStudy() {
        return study;
    }

    public List<MedicationSubstance> getMedications() {
        return medications;
    }

    @Override
    public void loadSemanticInformation() {
        if (bean.getCreationDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("creationDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getCreationDate().getTime()));
        }
        if (bean.getUpdateDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("editionDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getUpdateDate().getTime()));
        }
        if (bean.getStartDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("startDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getStartDate().getTime()));
        }
        if (bean.getEndDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("endDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getEndDate().getTime()));
        }
        //Object properties
        if (bean.getCreateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasCreationClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getUpdateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasEditionClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getPatientNdx() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPatient");
            classToOWL.createObjectProperty(individual, patientBean.getIndividual(), property);
        }
        //hasStudy
        //hasMedication
        for (MedicationSubstance mb : medications) {
            if (mb.getDesignation() != null) {
                String className = mb.getDesignation().replaceAll(" ", "");
                className = className.replaceAll(",", "_");
                if (className.startsWith("Interferonalfa_nat")) {
                    className = "Interferonalfa_naturlich";
                }
                OWLClass oc = classToOWL.getClass(className);
                classToOWL.createIndividual("Medication_" + mb.getSubstanceNdx(), mb.getSubstanceNdx().toString(), oc);
                OWLIndividual medInd = classToOWL.getIndividual("Medication_" + mb.getSubstanceNdx());
                //load medication individual
                //load designation
                OWLDataProperty designation = classToOWL.getDataProperty("designation");
                classToOWL.createStringDataProperty(designation, medInd, mb.getDesignation());
                if (mb.getSingleDose() != null) {
                    //load singleDose
                    OWLDataProperty singleDose = classToOWL.getDataProperty("singleDose");
                    classToOWL.createLongDataProperty(singleDose, medInd, mb.getSingleDose());
                }
                if (mb.getTotalDose() != null) {
                    //load totalDose
                    OWLDataProperty totalDose = classToOWL.getDataProperty("totalDose");
                    classToOWL.createLongDataProperty(totalDose, medInd, mb.getTotalDose());
                }
                if (mb.getUnit() != null) {
                    //load unit
                    OWLDataProperty unit = classToOWL.getDataProperty("unit");
                    classToOWL.createStringDataProperty(unit, medInd, mb.getUnit());
                }
                if (mb.getAdministration() != null) {
                    //load MedicationAdministration
                    OWLObjectProperty medAdm = classToOWL.getObjetProperty("hasMedicationAdministration");
                    OWLIndividual medAdmInd = classToOWL.getIndividual("MedicationAdministration_" + mb.getAdministration());
                    classToOWL.createObjectProperty(medInd, medAdmInd, medAdm);
                }
                //
                OWLObjectProperty hasMedication = classToOWL.getObjetProperty("hasMedication");
                classToOWL.createObjectProperty(individual, medInd, hasMedication);
            }
        }
    }

    public void loadSecondarySemanticInformation() {
        // Load the diagnosis
        for (DiagnosisBean db : patientBean.getDiagnosises()) {
            if (db.getBean().getDiagnosisNdx().longValue() == bean.getDianosisNdx().longValue()) {
                if (db.getIndividual() != null) {
                    OWLObjectProperty property = classToOWL.getObjetProperty("hasDiagnosis");
                    classToOWL.createObjectProperty(individual, db.getIndividual(), property);
                }
            }
        }
    }

    @Override
    public int compareTo(TherapyBean o) {
        return bean.getStartDate().compareTo(o.getBean().getStartDate());
    }

}
